#ifndef SPECMON_HH
#define SPECMON_HH

#include "OperStateCondList.hh"


#include <string>
#include <stream.h>
#include <fstream>
#include "gmutex.hh"

#include <TGFrame.h>
#include <TGButton.h>
#include <TGLabel.h>
#include <TRootEmbeddedCanvas.h>
#include <TH1.h>
#include "Histogram1.hh"
#include "ligogui/TLGChannelBox.hh"
#include "ligogui/TLGEntry.hh"
#include <TGNumberEntry.h>
#include <TGTextEntry.h>
#include <TPaveLabel.h>
#include <TSocket.h>
#include <TServerSocket.h>
#include <TMonitor.h>

#include "DatEnv.hh"
#include <TSeries.hh>
#include <FSeries.hh>
#include <vector>
#include "AlarmClient.hh"
#include "AlarmData.hh"
#include "MonServer.hh"
#include "TrigRslt.hh"
#include "TrigClient.hh"
#include <TPaveText.h>
#include <TText.h>

#include "Parser.hh"

static const size_t _24average = 86400;
static const int mDebugParseval =-1000;
static const int mDebugConfigfile =-2000;
static const int mDebugProblemActions = -3000;
static const int mDebugAll = -4000;
static const int mDebugAlarms = -5000;
static const int mDebugTriggers = -6000;
static const int mDebugTest = -7000;

using namespace std;

class BandInfo
{
public:
  //Constructor
  BandInfo();

  //Destructor
  ~BandInfo();

  //  void BandProblemActions(float freqlo,float freqhi, float powerlo, float powerhi,int bandnumber, int BandErrorCount,Time);
  
  string	BandName;
  double 	freqlo;
  double 	freqhi;
  double 	powerlo;
  double 	powerhi;
  int       	BandErrorCount_lo;
  int       	BandErrorCount_hi;
  Time      	LastProblemTime_lo;
  Time      	LastProblemTime_hi;
  size_t	LastProblemFrame_lo;
  size_t    	LastProblemFrame_hi;
  string 	AlarmDesc;
  bool 	    	hi_status;
  bool		lo_status;
  AlarmData 	AD;
  AlarmHandle	AH;
  double 	_24hr_mean;
  double        _12hr_mean;
  double	_6hr_mean;
  double	_1hr_mean;
  double        _30min_mean;
  double        _10min_mean;
  double        _1min_mean;
  vector<float>	mean_vec;
  vector<int>	triggers;
  vector<size_t>trigger_times;
  vector<Time>  trigger_times_t;
  double	trigger24_lo;//mean for 24hrs of low trigger
  double        trigger12_lo;//""
  double	trigger6_lo;
  double 	trigger1_lo;
  double	trigger24_hi;
  double        trigger12_hi;
  double	trigger6_hi;
  double 	trigger1_hi;
  double        trigger1min_lo;
  double        trigger1min_hi;
  double        trigger10min_lo;
  double        trigger10min_hi;
  double        trigger30min_lo;
  double        trigger30min_hi;
  double        maxPower;//highest power in x amount of time
  double        minPower;//lowest '' ''
  size_t        maxPowerFrame;//frame when max power occurred
  size_t        minPowerFrame;//"" min ""
};



class ChannelInfo
{
public:
	
  //Constructor
  ChannelInfo();
	
  //Destructor
  ~ChannelInfo();
	
  int BandNum;
  string ChannelName;
  vector<BandInfo> Bands;
	
};

class SpecMon;

/** @name Test monitor with graphical user interface
    
This monitor incoporates a graphical user interface.

/** Configuration information record.
    
@memo Configuration info
@author Daniel Sigg
***********************************************************************/
class GuiMonConf : public thread::mutex {
public:
  /// Initialized a configuration record
  GuiMonConf ();
  
  /// New configuration flag
  bool			fNew;
  /// Configuration valid?
  bool			fValid;
  /// Poreccing paused?
  bool			fPause;
  /// Channel name
  std::string		fChnName;
  /// X limits
  double			fXLimit[2];
  /// X Bins
  int			fXBins;
};

/** Data record.
    
@memo Data info
@author Daniel Sigg
***********************************************************************/
class GuiMonData : public thread::mutex {
public:
  /// Initialized a configuration record
  GuiMonData ();
  /// Destruct data object
  ~GuiMonData();
  
  /// New data flag
  bool			fNew;
  /// Histogram
  Histogram1		fHist;
};

/** Main window of Gui test monitor.
    
@memo SpecMon main window
***********************************************************************/
class SpecMonWindow : public TGMainFrame {
public:
  /// Create the main window
  SpecMonWindow (const TGWindow *p, UInt_t w, UInt_t h,
		 SpecMon& monitor, const char* chnnames);
  /// Destroy the main window
  virtual ~SpecMonWindow();
  /// Close the main window
  virtual void CloseWindow();
  /// Read configuration information
  virtual Bool_t ReadConf();
  /// Draw plot with new data
  virtual Bool_t DrawNew();
  //open the server socket
  virtual void Sockets(string type);
  //open server socket for update every x seconds
 
 // virtual void Sockets(string type, TTimer* timer);

  void GetStripChart(TMessage *mess);
  void GetHistogram(TMessage *mess);
  void GetMeanValues(TMessage *mess);
	
 //change the values in the BandSelect combobox
  void BandSelectUpdate(int channelnumber, int numbands);
  //Update Strip Chart
  void UpdateStripChart(vector<float> power);
  /// Process GUI messages
  virtual Bool_t ProcessMessage (Long_t msg, Long_t parm1, Long_t);
  /// Handle timer message, ie, display new data if ready
  virtual Bool_t HandleTimer (TTimer* timer);
  
  
  vector<float> power;
  TH1F *stripchart;  
  int NumberBandsToDelete;
  int fChannelSelected; //channel number that was selected.  referenced throughout the program
  int fBandSelected;
  Double_t fBinsSelected;
  
  //varialbles needed to read in the configuration file---------------------------------
  
  int mDebug;
  
  vector<string> info;
  vector<ChannelInfo>	Channels;
  
  int mChannelNumber;
		
  //---------------------------------------------------------------------------------------

  bool SendingData;//true if a socket is open. output a message to wait until the socket is finished


  TMonitor* mon;
  


  //StripChart *power;

protected:

  /// Monitor 
  SpecMon*		fMonitor;
  /// Configuration
  GuiMonConf*		fConf;
    /// Data
  GuiMonData*		fData;

  //stuff for testing a new band
  bool BandTestingMode;
  bool FirstUpdateTestBand;
  float newfreqlo;
  float newfreqhi;
  float newpowerlo;
  float newpowerhi;
  string newBandName;
  /*
  float freql;
  float freqh;
  float powerl;
  float powerh;
  */


  bool fSocketUse;
  
  bool bins;
 
  
  TH1* histogram;
  
  // Frames
  TGCompositeFrame *fFrame[30];
  
  
  TGCompositeFrame *fF[6];
  //TGCompositeFrame*		fFnew[13];
  
  TGCompositeFrame* fFramenew[47];
  TGLabel*  fLabelStats[36];

  /// Layout hints
  TGLayoutHints    *fL[28];
  
  
  /// Labels
  TGLabel *fLabel[18];
  TGLabel *fLabelNew[8];
  
  
  //Buttons------------------------

  TGComboBox *fChannelSelect;
  TGComboBox *fBandSelect;
  
  ligogui::TLGNumericControlBox *fFreq[2];//frequency selection boxes
  ligogui::TLGNumericControlBox *fPower[2];//power selection boxes
  ligogui::TLGNumericControlBox *fBins;//bins for historgram
  
  
  TGTextButton* fLogScale;
  TGTextButton* fLogScaleX;
  TGTextButton* fLogScaleStripY;
  TGTextButton* fLogScaleStripX;
  //TGCheckButton* fLogScale;

  TGTextButton* fExit;
  TGTextButton* fUpdate;
  //TGTextButton* fReset;
  //TGTextButton* fUpdateHistogram;

  /// Number entry for X limits
  ligogui::TLGNumericControlBox*	fXLimit[2];
  /// Number entry for X bins
  ligogui::TLGNumericControlBox*	fXBins;

  TCanvas*    fHistogram;
  TCanvas*    fStripChart;
  TCanvas*    fStatistics; 
  
  
  TServerSocket*  ss;
  
  bool logscale;
  bool logscalex;
  bool logscalestrip;
  bool logscalestripx;
      


  //stuff for displaying the stats      
  TPaveText* fStatPaveText;
  TText* mean1hr;
  TText* mean6hr;
  TText* mean12hr;
  TText* mean24hr;
  TText* triggerlo1hr;
  TText* triggerlo6hr;
  TText* triggerlo12hr;
  TText* triggerlo24hr;
  TText* triggerhi1hr;
  TText* triggerhi6hr;
  TText* triggerhi12hr;
  TText* triggerhi24hr;

      
      
      
  //TPaveText		*fTriggerInfo;
  
  
  /// Embedded plot pad
  TRootEmbeddedCanvas*	fPlot[4];
  
  
  /// Pause/resume/exit buttons
  TGButton*			fButton[4];
  /// Heartbeat timer
  TTimer*			fHeartbeat;
  /// Skip Heartbeats
  Int_t			fSkipHeartbeats;
  /// 1D Histogram
  TH1D			fHist;
  
  /// Button foreground context
  static GContext_t 	fgButtonGC;
  /// Button font
  static FontStruct_t 	fgButtonFont;
};

/** SpecMon is the object implementing the main program
    
@memo SpecMon main program
@author Tim Bodiya
***********************************************************************/
class SpecMon : 
  public DatEnv,
  public MonServer,
  private TrigClient
{
  /// The main window is a friend
  friend class SpecMonWindow;
  
public:
  /// Construct monitor
  SpecMon (int argc, const char* argv[]);
  /// Desctruct monitor
  virtual ~SpecMon();
  /// Process data
  virtual void ProcessData();
  int               mDebug;

private:
  
  // Name of monitor
  std::string       mName;
  
  //void              appendToLog(Time now);
  //void              updatePowerWindows(Time now);
    
  //__________________________________________Process Data Stuff____________
  vector<ChannelInfo>	Channels;
  
  void FillMeanHistogram(TH1S* mean,size_t framecount,int bandnumber, int channelnumber);

  TH1F* CreateStripChart(size_t framecount,int channelnumber,int bandnumber,string name, double &min, double &max);
  
  TH1F* CreateHistogram(size_t framecount,int channelnumber, int bandnumber, string name, double min, double max, int numbins);

  void BackgroundSocket(size_t framecount);
  void AddBand(int channelnumber, int bandnumber, double freqlo, double freqhi, double powerlo, double powerhi);
  void DeleteBand(int channelnumber);

  //Trigger Types and Subtypes
  string  mTrigType;
    
  bool mTestBand;
  int  mTestChannel;
  int mChannelTest;
  int mBandTest;
  
  //  Number of frames I should process
  size_t            MaxFrame;    
  
  //Frame # being processed
  size_t            mFrameCount; 
  size_t            mFrameCount_Test;//counts the # of frames from start of the test band
  
  //Ifo name
  string       mIFOname;
      
  /// Log file
  ofstream          mLogfile;     
  string            mLogfileName;
  ostream*          mLog;   
  int	            mChannelNumber;
  int               dtotalbands;
  vector<string>    dLogfilename;
  ofstream          dLogfile;
  
  bool mFork;

  int guibins;
  
  /// Name of status display file
  std::string   mDispfileName;

  //Number of Time the FindDeviations loop is run
  int  mAccuracy;
  
  OperStateCondList mOsclist;
 
  void SetBools(bool &one,bool &ten, bool &thirty, bool &hour, bool &sixh, bool &twelveh, bool &day, int framecount);
  vector<int> GetTriggerInfo(int channelnumber, int bandnumber, int mFramecount);
  
  void  BandProblemActions(vector<ChannelInfo> &bChannels,int bChannelNumber, int bBandNumber,Time timeNow,bool testband,vector<int> &btriggers,vector<size_t> &btrigger_times,vector<Time> &btrigger_times_t);
  
  TH1D  Spectrum(FSeries*& fs);

  //TSeries to fill in the FSeries 
  TSeries*   mTSeries;
   
  //FSeries
  FSeries*   mFSeries_temp;
   
  //Stuff from ServoMon 
    
  // Total time that monitor has been running
  Interval      mTotalRuntime; 
    
  //
  // Alarm stuff
  //
  /// Alarm client
  AlarmClient       mAlarmClient;
  
  void SetAlarmDesc(string urldir,vector<ChannelInfo> &Channels,int ChannelNumber);
  
  /// HTML directory
  std::string       mHtmlDir;
 
  /// name of index page
  std::string       mIndexPageName;
  
  /// index page
  std::ofstream     mIndexPage;
  
  ///////////////////////////////////////////////////////////////
  //socket stuff
  
  TSocket *socket;

  ////////////////////////////////////////////////////////////////////////////////////////

protected:
  /// Configuration
  GuiMonConf	fConf;
  /// Data
  GuiMonData	fData;
  /// Pointer to main window
  SpecMonWindow*	fMainWindow;
  /// First time around?
  bool		fFirstTime;
  /// Time series object 
  TSeries*		fTS;
};

//@}
#endif


























































