#include <Rectangle.h>
#include <TidClass.h>
#include <math.h>

void TidRectangle::SlicesUndefined () {

  for(unsigned int i=0; i<SlicesDefined.size(); i++)
    SlicesDefined[i] = 0;

}

double TidRectangle::Area(vector<double>* par, double bpp) {

  double nb = ((*par)[1] - (*par)[0]) * ((*par)[3] - (*par)[2]),
    nw = ((*par)[7] - (*par)[6]) * ((*par)[5] - (*par)[4]);

  return nb*(1.0-bpp) + bpp*(nw-nb);
}


double TidRectangle::CostFunction(ArrayBool *data, vector<double>* par) {

  unsigned int io;

  double cost = 0.0;

  ArrayBool::iterator da;
  unsigned int ti;

  bool fBlack = 1;

  //  cout<<paramOpti.size()<<endl;

  // set io to paramOpti matching par
  for(io=0; io < paramOpti.size(); io++) {
    unsigned int k;
    for(k=0;k<par->size();k++)
      if((paramOpti[io])[k] != (*par)[k]) break;
    if(k == par->size()) break;
  }


  if((io && io == paramOpti.size()) || !paramOpti.size()) {
    SlicesDefined.push_back(0);
    lastCost.push_back(0);
    firstWhite.push_back(0);
    firstBlack.push_back(0);
    oalastBlack.push_back(0);
    paramOpti.push_back(*par);
    
    io = paramOpti.size() - 1;
  }

  if(!SlicesDefined[io]) {
    
    firstWhite[io] = firstBlack[io] = oalastBlack[io] = 0.0;
    bool fWDef = 0;

  for(ti=0, da=data->begin(); 
      ti<(unsigned int)floor((*par)[5]/TimeResolution) &&
	da!=data->end();
      ti++, da++) {

    double dcost = 0.0;

    // loop over black
    if(ti >= (unsigned int)floor((*par)[0]/TimeResolution) &&
       ti < (unsigned int)floor((*par)[1]/TimeResolution)) {

      // loop over white
      for(unsigned int fi=(unsigned int)floor((*par)[6]*TimeResolution);
	  fi < (unsigned int)floor((*par)[2]*TimeResolution);
	  fi++)
	if((*da)[fi]) dcost += alpha;

      for(unsigned int fi=(unsigned int)floor((*par)[3]*TimeResolution);
	  fi < (unsigned int)floor((*par)[7]*TimeResolution);
	  fi++)
	if((*da)[fi]) dcost += alpha;

      // loop over black
      for(unsigned int fi=(unsigned int)floor((*par)[2]*TimeResolution);
	  fi < (unsigned int)floor((*par)[3]*TimeResolution);
	  fi++)
	if(!(*da)[fi]) dcost += 1.0;

      
      if(fBlack) {firstBlack[io] = dcost; fBlack = 0;}

    } // time check if black 
    else {

      if((ti < (unsigned int)floor((*par)[0]/TimeResolution) &&
	  ti >= (unsigned int)floor((*par)[4]/TimeResolution)) ||
	 (ti >= (unsigned int)floor((*par)[1]/TimeResolution) &&
	  ti < (unsigned int)floor((*par)[5]/TimeResolution))) {

	// loop over white
	for(unsigned int fi=(unsigned int)floor((*par)[6]*TimeResolution);
	    fi < (unsigned int)floor((*par)[7]*TimeResolution);
	    fi++)
	  if((*da)[fi]) dcost += alpha;

	if(!fWDef) {fWDef = 1; firstWhite[io] = dcost;}
	
	if(!fBlack) {oalastBlack[io] = dcost; fBlack = 1;}
      }
    }

    cost += dcost;
  }

  SlicesDefined[io] = 1;

  }
  else { // if not first time

    unsigned int fBid, fWid;
    double fW, obfB, fB, lB, oalB, lW;
    fW = obfB = fB = lB = oalB = lW = 0.0;

    // first white
    fWid=ti=(unsigned int)floor((*par)[4]/TimeResolution);
    da=data->begin();
    for(unsigned int k=0;k<ti;k++) da++;

    if(ti < (unsigned int)floor((*par)[0]/TimeResolution)) {
      for(unsigned int fi=(unsigned int)floor((*par)[6]*TimeResolution);
	  fi < (unsigned int)floor((*par)[7]*TimeResolution);
	  fi++)
	if((*da)[fi]) fW += alpha;
    }

    // find first black
    for(ti=0, da=data->begin(); 
	ti<(unsigned int)floor((*par)[5]/TimeResolution) &&
	da!=data->end();
	ti++, da++) 
      if(ti >= (unsigned int)floor((*par)[0]/TimeResolution))
	break;
    fBid = ti;

    // do one before first black
    if(da!=data->begin() && 
       ti> (unsigned int)floor((*par)[4]/TimeResolution)) {

      da--; ti--;

      // check not first white or first black
      if(ti!=fWid && ti !=fBid) {
	for(unsigned int fi=(unsigned int)floor((*par)[6]*TimeResolution);
	    fi < (unsigned int)floor((*par)[7]*TimeResolution);
	    fi++)
	  if((*da)[fi]) obfB += alpha;
      }

      da++; ti++;

    }
    else obfB = fW;

    // do first black
    if(da!=data->end()) {

      // loop over white
      for(unsigned int fi=(unsigned int)floor((*par)[6]*TimeResolution);
	  fi < (unsigned int)floor((*par)[2]*TimeResolution);
	  fi++)
	if((*da)[fi]) fB += alpha;

      for(unsigned int fi=(unsigned int)floor((*par)[3]*TimeResolution);
	  fi < (unsigned int)floor((*par)[7]*TimeResolution);
	  fi++)
	if((*da)[fi]) fB += alpha;

      // loop over black
      for(unsigned int fi=(unsigned int)floor((*par)[2]*TimeResolution);
	  fi < (unsigned int)floor((*par)[3]*TimeResolution);
	  fi++)
	if(!(*da)[fi]) fB += 1.0;

    }
    unsigned int lti = ti; // first black

    // find one after last black
    for(;
	ti<(unsigned int)floor((*par)[5]/TimeResolution) &&
	  da!=data->end();
	ti++, da++) 
      if(ti >= (unsigned int)floor((*par)[1]/TimeResolution))
	break;
    
    // do last black if not first black
    if(ti > lti + 1 && da != data->begin()) {
      ti--; da--;

      // loop over white
      for(unsigned int fi=(unsigned int)floor((*par)[6]*TimeResolution);
	  fi < (unsigned int)floor((*par)[2]*TimeResolution);
	  fi++)
	if((*da)[fi]) lB += alpha;

      for(unsigned int fi=(unsigned int)floor((*par)[3]*TimeResolution);
	  fi < (unsigned int)floor((*par)[7]*TimeResolution);
	  fi++)
	if((*da)[fi]) lB += alpha;

      // loop over black
      for(unsigned int fi=(unsigned int)floor((*par)[2]*TimeResolution);
	  fi < (unsigned int)floor((*par)[3]*TimeResolution);
	  fi++)
	if(!(*da)[fi]) lB += 1.0;

      ti++; da++;
    }
    else lB = fB;
    

    // do one after last black
    if(da!=data->end() &&
       ti<(unsigned int)floor((*par)[5]/TimeResolution) &&
       ti>=(unsigned int)floor((*par)[1]/TimeResolution)) {

      for(unsigned int fi=(unsigned int)floor((*par)[6]*TimeResolution);
	  fi < (unsigned int)floor((*par)[7]*TimeResolution);
	  fi++)
	if((*da)[fi]) oalB += alpha;

    }
    lti = ti; // one after last black

    // finally, do last white
    for(; 
	ti<(unsigned int)floor((*par)[5]/TimeResolution) &&
	  da!=data->end();
	ti++, da++) {}
    if(da!=data->begin() && 
       ti > lti+1 &&
       ti>=(unsigned int)floor((*par)[1]/TimeResolution)) {
      da--; ti--;

      for(unsigned int fi=(unsigned int)floor((*par)[6]*TimeResolution);
	  fi < (unsigned int)floor((*par)[7]*TimeResolution);
	  fi++)
	if((*da)[fi]) lW += alpha;
    }
    else lW = oalB;

    // compute new cost
    //    printf("%i:\t%g\t%g\t%g\t%g\t%g\t%g\n",io,fW,obfB,fB,lB,oalB,lW);
    cost = lastCost[io] - firstWhite[io] - firstBlack[io] + obfB - oalastBlack[io] + lB + lW;

    firstWhite[io] = fW;
    firstBlack[io] = fB;
    oalastBlack[io] = oalB;
  }

  lastCost[io] = cost;

  return cost;
}



void TidRectangle::Delete(ArrayBool* data, vector<double>* par) {

  ArrayBool::iterator da;
  unsigned int ti;

  for(ti=0, da=data->begin(); 
      ti<(unsigned int)floor(((*par)[5]-(*par)[4])/TimeResolution) &&
	da!=data->end();
      ti++, da++) 
    for(unsigned int fi=(unsigned int)floor((*par)[6]*TimeResolution);
	fi < (unsigned int)floor((*par)[7]*TimeResolution);
	fi++)
      (*da)[fi] = 0;

  SlicesUndefined();
}


/****************************************************************/

double TidCompositeRectangle::GetMaxT () {

  double mint,maxt;

  list<TidRectangle>::iterator p=rectangles.begin();

  mint = (*p).GetParamMin(4);
  maxt = (*p).GetParamMax(5);

  for(p++; p!=rectangles.end(); p++) {
    if(mint > (*p).GetParamMin(4)) mint = (*p).GetParamMin(4);
    if(maxt < (*p).GetParamMax(5)) maxt = (*p).GetParamMax(5);
  }
  
  return maxt - mint;

}


bool TidCompositeRectangle::Test(ArrayBool *data, double maxCost, vector<string>* outpar, double *evDuration, double T0, double BPP, bool NormalizeCost) {

  bool DoIt;
  vector<double> varVal = varlb;
  unsigned int nVar = varVal.size();
  double cost = 0.0;
  double area;


  DoIt=1;
  bool oneAboveThr=0;
    
  while(DoIt) {

    cost = 0.0;
    area = 0.0;

    // compute cost with actual parameters

    list<TidRectangle>::iterator rec;
    list<VectorUInt>::iterator par;
    double dura = 0.0;

    for(rec=rectangles.begin(), par=params.begin(); 
	rec!=rectangles.end(); 
	rec++, par++) {

      vector<double> tmp;

      for(unsigned int i=0; i<8; i++) {
	if((*par)[i] < 1000) tmp.push_back((*rec).GetConstant((*par)[i]));
	else {
	  //	  printf("*%g\t\t%g\t\t%i\t\t%g*\n",(*rec).GetAdd(i),(*rec).GetMul(i),(*par)[i], varVal[(*par)[i]-1000]);
	  tmp.push_back((*rec).GetAdd(i) + (*rec).GetMul(i) * varVal[(*par)[i]-1000]);
	}

	//	cout<<tmp[i]<<"\t"<<flush;
      }

      //      cout<<endl;

      if(tmp[1] - tmp[0] > dura) dura = tmp[1] - tmp[0];

      double dcost = (*rec).CostFunction(data, &tmp);

      area += (*rec).Area(&tmp, BPP);

      cost+=dcost;

      //      cout<<"\n"<<dcost<<"\n";

      tmp.clear();
    } // loop over rectangles

    //    cout<<cost<<endl;

    if(NormalizeCost) cost /= area;

    if(Debug) {
      printf("Variables:\n");
      for(unsigned j=0; j<varname.size(); j++)
	printf("%c = %g\t\t",varname[j], varVal[j]);
      printf("\nTime:%.15g\t\tCost: %g\n.......................oooOOOooo........................\n", T0, cost);


      if(dFileOpen) {
	fprintf(dFile,"%.15g\t",T0);
	for(unsigned j=0; j<varname.size(); j++)
	  fprintf(dFile,"%g\t",varVal[j]);
	fprintf(dFile,"%g\n",cost);
      }
    }



    if(cost < maxCost) {

      inEvent=1;
      oneAboveThr=1;
      nAboveThr=0;

      if(cost < minCost) {
	minTime = T0;
	minCost=cost;
	varBest = varVal;
	evDur = dura;
      }

    }

    // increment value
    if(nVar) {
      varVal[nVar-1] += dvar[nVar-1];

      for(int i = nVar-1; i>=0; i--) {
	if(varVal[i] > varub[i]) {
	
	  if(i==0) {DoIt=0; break;}
   
	  varVal[i] = varlb[i];
	  varVal[i-1] += dvar[i-1];

	}
	else break;
      }
    }
    else {
      DoIt = 0;
    }

  }

  if(!oneAboveThr && inEvent) {
    
    nAboveThr++;

    // test
    if(nAboveThr > maxAboveThrStop) {

      *evDuration = evDur;

      // form output vector
      char buf[256];
      sprintf(buf,"%g",minCost);
      (*outpar).push_back(string(buf));
      sprintf(buf,"%.15g",minTime);
      (*outpar).push_back(string(buf));
      for(unsigned int i=0; i<varname.size(); i++) {
	sprintf(buf,"%c",varname[i]);
	(*outpar).push_back(string(buf));
	sprintf(buf,"%g",varBest[i]);
	(*outpar).push_back(string(buf));
      }


      /*
      // remove winner from list
      list<TidRectangle>::iterator rec;
      list<VectorUInt>::iterator par;
      varVal = varBest;
      for(rec=rectangles.begin(), par=params.begin(); 
	  rec!=rectangles.end(); 
	  rec++, par++) {

	vector<double> tmp;
	
	for(unsigned int i=0; i<8; i++) {
	  if((*par)[i] < 1000) tmp.push_back((*rec).GetConstant((*par)[i]));
	  else {
	    tmp.push_back((*rec).GetAdd(i) + (*rec).GetMul(i) * varVal[(*par)[i]-1000]);
	  }
	}

	(*rec).Delete(data, &tmp);

	tmp.clear();
      }
      */

      // reset
      inEvent = 0;
      minCost = 1E100;
      nAboveThr=0;      

    return 1;
    }
  }

  return 0;
}




void TidCompositeRectangle::PushBack(TidRectangle r) {

  // add rectangle
  rectangles.push_back(r);

  // update variable list
  vector<unsigned int> tmp = r.GetParams();
  unsigned int i,j;

  for(i=0; i<tmp.size(); i++) 
    if(tmp[i] >= 1000) {

      for(j=0; j<varname.size(); j++)
	if(varname[j] == (*r.GetVarName())[tmp[i]-1000])
	  break;
	
      if(j==varname.size()) {
	varname.push_back((*r.GetVarName())[tmp[i]-1000]);
	varlb.push_back((*r.GetVarLB())[tmp[i]-1000]);
	varub.push_back((*r.GetVarUB())[tmp[i]-1000]);
	dvar.push_back((*r.GetDVar())[tmp[i]-1000]);
      
	// redo the binding
	tmp[i] = 1000 + dvar.size()-1;
      }
      else { // redo the binding
	tmp[i] = 1000 + j;
      }
    }

  params.push_back(tmp);
}



TidCompositeRectangle::~TidCompositeRectangle () {

  if(dFileOpen) fclose(dFile);

}


void TidCompositeRectangle::SetDebugFile(string file) {

  DebugFile = file;

  if(!(dFile = fopen(file.c_str(),"w"))) {
    throw TidError (string("Unable to open file ") + file + string(" for writing."));
  }

  dFileOpen = 1;

  fprintf(dFile,"# time\t ");
  for(unsigned j=0; j<varname.size(); j++)
	fprintf(dFile,"%c\t",varname[j]);
  fprintf(dFile,"cost\n");
}
    

		    

void TidCompositeRectangle::Dump(FILE *ou) {

  bool DoIt=1;
  vector<double> varVal = varlb;
  unsigned int nVar = varVal.size();

  list<TidRectangle>::iterator rec;
  list<VectorUInt>::iterator par;


  while(DoIt) {

    for(unsigned int k=0; k<nVar; k++) {
      fprintf(ou,"%c\t%g\t",varname[k],varVal[k]);
    }
    fprintf(ou,"\n");

    for(rec=rectangles.begin(), par=params.begin(); 
	rec!=rectangles.end(); 
	rec++, par++) {

      double tmp;

      for(unsigned int i=0; i<8; i++) {
	if((*par)[i] < 1000) tmp = (*rec).GetConstant((*par)[i]);
	else {
	  tmp = (*rec).GetAdd(i) + (*rec).GetMul(i) * varVal[(*par)[i]-1000];
	}

	fprintf(ou,"%g\t",tmp);
      }
      fprintf(ou,"\n");
    }

        if(nVar) {
      varVal[nVar-1] += dvar[nVar-1];

      for(int i = nVar-1; i>=0; i--) {
	if(varVal[i] > varub[i]) {
	
	  if(i==0) {DoIt=0; break;}
   
	  varVal[i] = varlb[i];
	  varVal[i-1] += dvar[i-1];

	}
	else break;
      }
    }
    else {
      DoIt = 0;
    }
  }
}
