#ifndef TID_H
#define TID_H 1

#include <stdlib.h>
#include <stdio.h>

#include <signal.h>

#include <iostream>
// #include <sstream>

#include <string>
#include <vector>
#include <queue>
#include <list>
#include <stack>

//DMT
#include <Dacc.hh>
#include <Time.hh>
#include <TSeries.hh>
#include <FSpectrum.hh>
//#include <Filter.hh>

// TID
#include <errors.h>
#include <event.h>
#include <channel.h>
#include <TidIO.h>
#include <request.h>
#include <TidString.h>
#include <OperStateCondList.hh>

class Tid {  // main class, includes parameters and data

  bool ContinueLoop; // set to 0 to exit tid::Loop

  OperStateCondList* osclist;

  std::list<std::string> oscConditions;

  bool DMTBackGround; // set to 1 if running under DMT manager

  std::string sName; // server name

  Interval tres; // time resolution

  Time stime; // starting time

  Time etime; // ending time

  bool T0Set;
  Time T0; // starting time corresponding to time id = 0


  std::list<unsigned int> cidlist; // list of channel Id's 

  std::list<ChnInfo*> diag; // diagnostic channels descriptions

  std::list<ChnInfo*> ref; // reference channels



  std::string source; // source of data

  Dacc *In; // dacc object, source of frame data

  std::list<TSeries**> data; // pointer to data in time series


  TidIO* IO; // IO server object for data output

  TidBroadcast *broadcaster; // server for output

  
  TidIO* IOin; // IO server object for communication


  TidRequest requests; // buffer for requests from clients (input)

  

  std::queue<std::string> messages;  // buffer of messages from server

  std::queue<std::string> complains; // buffer for errors/warnings/comments


  std::stack<Event*> ePool; // Pool of pointers to preallocated events

  unsigned int maxClusterSize; // default = 30

 public:

  unsigned int GetmaxClusterSize() {return maxClusterSize;}

  void TidCon ();

  Tid (); // constructor

  Tid (Dacc *);

  void SetParamsFromFile(FILE*); // default parameters

  void WriteParamsToFile(FILE*); // default parameters

  void AddChannel(string chnName,
		  unsigned int sigma,
		  vector<unsigned int>& delta,
		  float alpha,
		  float BPP_,
		  unsigned int fs_,
		  double fLow_,
		  double fHigh_,
		  unsigned int dec_,
		  Interval deltaT_,
		  Interval stabScale_,
		  double stabTol_,
		  vector<unsigned int>& testFreq_,
		  bool OSC_ = 0);
  
  unsigned int GetSamplingByName(const string&);

  void AddMessage (const string& s);

  void AddComplain(const string& s);

  bool Loop (); // main computation loop

  void SendEvent (Event *p) {broadcaster->SendEvent(p);} // Event sender

  ~Tid(); // destructor

  std::string GetsName () {return sName;}
  std::string GetSource () {return source;}
  double GetTRes () { return (double)tres;}

  Event* NewEvent (); // return pointer to Event

  void DeleteEvent (Event*); // clean Event and put pointer back into pool

  std::list<ChnInfo*>::iterator GetDiagBegin () {return diag.begin();}
  std::list<ChnInfo*>::iterator GetDiagEnd () {return diag.end();}

  void ProcessInstructions(TidString &); // process string instructions

  double GetT0 () {return (double)T0.getS() + 1E-9*(double)T0.getN();}

  void InterruptLoop() { ContinueLoop = 0; }

  bool GetDMTBackground () {return DMTBackGround;}
};

#ifdef _TIDG_
extern Tid* tidg; // main object
#endif


#endif








